#!/usr/bin/python
# coding=utf-8

import sys
mswindows = (sys.platform[:3] == 'win')

try:
	import pygtk
	pygtk.require("2.0")
except:
	print "pygtk 2.0 is not installed"
  
try:
	import gtk
	import gtk.glade
except:
	print "gtk and/or glade is not installed - exiting..."
	sys.exit(1)

import gettext
import locale
import os
import imp
import glob
basedir = '.'

try:
	basedir = os.path.dirname(os.path.realpath(__file__))
	prefix = os.path.abspath( os.path.normpath( os.path.join( basedir, '..')))
except:
	if hasattr(sys, 'frozen'):
		basedir = os.path.split(sys.executable)[0]
		prefix = os.path.abspath( os.path.normpath( basedir ) )
	else:
		basedir = os.path.split(sys.argv[0])[0] 
		prefix = os.path.abspath( os.path.normpath( os.path.join( basedir, '..')))
try:
	sys.path.remove(basedir)
except:
	pass

srcdir = os.path.join( prefix, 'src')
if os.path.exists(srcdir) and srcdir not in sys.path:
	sys.path.insert(0, srcdir)
else:
	webmarxdir = os.path.join( prefix, 'lib')
	if os.path.exists(webmarxdir) and webmarxdir not in sys.path:
		sys.path.insert(0, webmarxdir)	

import webmarx
DIR = os.path.join( prefix, 'share', 'locale')
from webmarx import settings
APP = 'webmarx'

# Init the list of languages to support
langs = []
#Check the default locale
lc, encoding = locale.getdefaultlocale()
if (lc):
	#If we have a default, it's the first in the list
	langs = [lc]
	
# Now lets get all of the supported languages on the system
language = os.environ.get('LANGUAGE', None)
if (language):
	langs += language.split(":")
langs += ["de_DE"]
locale.setlocale(locale.LC_ALL, '')
gettext.bindtextdomain(APP, DIR)
gettext.textdomain(APP)
gtk.glade.bindtextdomain(APP, DIR)
gtk.glade.textdomain(APP)

# Get the language to use
this_language = gettext.translation(APP, DIR
	, languages=langs, fallback = True)

gettext.install(APP, DIR, unicode=1)
global _
_ = this_language.gettext

## Set the global gettext translation for webmarx.
## NEVER REMOVE THIS, ELSE TRANSLATIONS WILL BE BROKEN!
webmarx._ = _

from webmarx.TextBufferMarkup import PangoBuffer

from webmarx.tools import Autoconnect, GTK_tools, DelayedAction, RunExternalApp
from webmarx.StaticPlugins import PluginLaunchbutton
from webmarx.tags import Tags
from webmarx.bookmarks import Bookmarks, Bookmarkhelper

from webmarx import Dialogs
from webmarx import trayapp


encoding = locale.getpreferredencoding()
utf8conv = lambda x : unicode(x, encoding).encode('utf8')

from webmarx.gui import Window, WindowSettings, BookmarkExpander, TagTable

class Webmarks:
	def __init__(self):
		self.backup_warning = False
		self.prefix = prefix
		self.lang = this_language
		self.last_id = 0
		self.Settings = settings.Settings(self)
		self.Settings.load()
		self.Bookmarks = Bookmarks(self, backup_warn_func=self.get_backup_warning)
		self.selected_bookmark = None
		self.Tags = Tags(self)
		self.Tags.load()
		self.launcherPlugins = None
		self.importPlugins = None
		self.exportPlugins = None
		self.loadPlugins()
		self.Window = None
		self.Tray = None
		
	def get_backup_warning(self):
		self.backup_warning = True

	def loadTray(self):
		self.Tray = trayapp.tray(self)
		
	def load_gui(self): 
		self.Window = Window(self)
		self.Window.Show()	   

	def loadPlugins(self):
		''' getting all plugin module-names and append them to the list'''
		self.launcherPlugins = None
		self.importPlugins = None
		self.exportPlugins = None
		tempimport = {}
		Defaultbrowser = None
		Defaultapp = None	
		data = []
		for path in self.Settings.plugindirs:
			data+=glob.glob(path + '/*.py')
		data.sort()
		prefixes = ('import','export','launch')
		for entry in data:
			filename = os.path.basename(entry)
			path = entry.replace(filename, '')
			if (os.path.isfile(entry)
			and filename.startswith(prefixes) 
			and filename.endswith('py')):
				file = open(entry, "r")
				imp.acquire_lock()
				try:
					module = imp.load_source( filename[0:-3], entry, file )
				finally:
					file.close()
				imp.release_lock()
				plugin = module.Plugin()
				if plugin.enabled == True:
					if filename.startswith('launch'):
						try:
							self.launcherPlugins.append(plugin)
						except:
							self.launcherPlugins = [plugin,]
						if filename == 'launchDefaultbrowser.py':
							Defaultbrowser = plugin
						elif filename == 'launchDefaultapp.py':
							Defaultapp = plugin
					elif filename.startswith('import'):
						try:
							self.importPlugins.append(plugin)
						except:
							self.importPlugins = [plugin,]
					elif filename.startswith('export'):
						try:
							self.exportPlugins.append(plugin)
						except:
							self.exportPlugins = [plugin,]
		if Defaultbrowser:
			self.launcherPlugins.remove(Defaultbrowser)
			self.launcherPlugins.insert(0, Defaultbrowser)
		if Defaultapp:
			self.launcherPlugins.remove(Defaultapp)
			self.launcherPlugins.insert(1, Defaultapp)
		return True	
		
	def exit(self, *args):
		gtk.main_quit()
		return False

if __name__ == "__main__":
	app = Webmarks()
	app.loadTray()
	app.load_gui()	
	gtk.main()	 
		
		
