#!/usr/bin/env python
# coding=utf-8

## (c) 2009-2012 by createweb - rainer haage, all rights reserved.
##	The Dialogs library is free software; you can redistribute it and/or
##	modify it under the terms of the GNU Lesser General Public License
##	as published by the Free Software Foundation; either version 2
##	of the License, or (at your option) any later version.
##	
##	This program is distributed in the hope that it will be useful,
##	but WITHOUT ANY WARRANTY; without even the implied warranty of
##	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##	GNU General Public License for more details.
##	
##	You should have received a copy of the GNU Lesser General Public License
##	along with this program; if not, write to the Free Software
##	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from os import path
import gettext
import gobject
import gtk
import locale
import os
import pango
import pickle
import pygtk  
import re
import shutil
import string
import sys
import time

#_ = gettext.gettext

tooltips = gtk.Tooltips()


class Dialog:
	"""
	 # PyUML: Do not remove this line! # XMI_ID:_cPV7WC3LEd21yqQLVqxUKg
	"""
	def __init__ (self, text, title=None, longtext=False, returnfunc=None, return_args=[], cancel_func=None, cancel_args=[]):
		''' Initialize method of the dialog 
		
		type	-- The dialog type as defined below. This has to be one of the
		following string-values: info, message, warning, error, request, apply1,
		apply2. For now there is no difference between apply1 and apply2.
		
		text	-- The text-string of the dialog
		
		title	-- If given, this will be used as window-title.
		
		returnfunc	-- An optional method that will be called on every form of
		positive request answers like ok, apply, confirm etc.
		
		return_args	-- optional arguments for the returnfunc-method.
		
		'''
		#self.returnfunc = None
		self.dialog = gtk.Dialog(title=None, parent=None, flags=0, buttons=None)
		self.dialog.set_keep_above(True)
		self.dialog.set_modal(True)
		self.dialog.connect('response', self.on_dialog_universal_response)
		self.image_messageitem = gtk.Image()
		self.maincontainer = gtk.HBox()
		self.dialog.vbox.pack_start(self.maincontainer, expand=True, fill=True, padding=0)
		self.maincontainer.pack_start(self.image_messageitem, expand=False, fill=True, padding=2)
		self.textview_longtext = None
		self.label_messagetext = None
		self.messagesbuffer = None
		self.scrolledwindow_longtext = None
		self.entryValue = None
		  
		self.types = ['info','message','warning','error','request','apply1','apply2','getstring']
		self.text = text
		self.title = title
		self.longtext = longtext
		self.returnfunc = returnfunc
		self.return_args = return_args
		self.cancel_func = cancel_func
		self.cancel_args = cancel_args		
		
		if self.title != None:
			self.dialog.set_title(self.title)
		
		if longtext == False:
			self.setup_label_messagetext()		 
		else:
			self.setup_textview_longtext()
			
	def set_modal(self, modal=False):
		self.dialog.set_modal(modal)		

			
	def setup_label_messagetext(self):
		self.label_messagetext = gtk.Label()
		self.label_messagetext.set_use_markup(True)
		self.label_messagetext.set_justify(gtk.JUSTIFY_LEFT)
		self.label_messagetext.set_markup(self.text)
		self.maincontainer.pack_start(self.label_messagetext, expand=True, fill=True, padding=2)
		self.label_messagetext.queue_resize()		
		return self.label_messagetext
		
	def setup_textview_longtext(self):
		self.messagesbuffer = gtk.TextBuffer()
		self.textview_longtext = gtk.TextView(buffer=self.messagesbuffer)
		self.messagesbuffer = self.textview_longtext.get_buffer()
		self.messagesbuffer.set_text(self.text) 
		self.scrolledwindow_longtext = gtk.ScrolledWindow(hadjustment=None, vadjustment=None)
		self.scrolledwindow_longtext.add(self.textview_longtext)
		self.maincontainer.pack_start(self.scrolledwindow_longtext, expand=True, fill=True, padding=2)
		self.textview_longtext.queue_resize()
		self.textview_longtext.set_editable(False)			
		self.textview_longtext.show_all() 
		self.dialog.set_size_request(500, 350)  
		return self.textview_longtext
  
		
	def on_dialog_universal_response(self, widget, response, *entry):	  
		if response == gtk.RESPONSE_APPLY or response == gtk.RESPONSE_OK:
			if self.returnfunc != None:
				self.returnfunc(self.return_args)
				time.sleep(0.1) 
			if response == gtk.RESPONSE_OK:
				widget.destroy ()
			return True		
		elif response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_CLOSE:
			if response == gtk.RESPONSE_CANCEL:
				if self.cancel_func != None:
					self.cancel_func(self.cancel_args)
					time.sleep(0.1)
			widget.destroy ()
			return False
		
class DialogInfo(Dialog):
	''' Default info dialog '''
	def __init__(self, text, title=None, longtext=False):
		''' init dialog
		
		text	-- The message text
		title	-- The window title
		longtext	-- boolean, if True, the text will be displayed in multi-line mode
		
		'''		   
		Dialog.__init__(self, text=text, title=title, longtext=longtext)
		self.dialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)

		self.image_messageitem.set_from_stock(gtk.STOCK_INFO, gtk.ICON_SIZE_DIALOG)
		self.dialog.show_all()	   
	
class DialogMessage(Dialog):
	''' Default message dialog '''
	def __init__(self, text, title=None, longtext=False):
		''' init dialog
		
		text	-- The message text
		title	-- The window title
		longtext	-- boolean, if True, the text will be displayed in multi-line mode
		
		'''		
		Dialog.__init__(self, text=text, title=title, longtext=longtext)
		self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
		self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_DIALOG)
		self.dialog.show_all()
		
		
		
class DialogWarning(Dialog):
	''' Default warning dialog '''
	def __init__(self, text, title=None, longtext=False):
		''' init dialog
		
		text	-- The message text
		title	-- The window title
		longtext	-- boolean, if True, the text will be displayed in multi-line mode
		
		'''		 
		Dialog.__init__(self, text=text, title=title, longtext=longtext)
		self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
		self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_DIALOG)
		self.dialog.show_all()
		
class DialogError(Dialog):
	''' Default error dialog '''
	def __init__(self, text, title=None, longtext=False):
		''' init dialog
		
		text	-- The text that describes the error
		title	-- The window title
		longtext	-- boolean, if True, the text will be displayed in multi-line mode
		
		'''
		self.returnfunc = None
		Dialog.__init__(self, text=text, title=title, longtext=longtext)
		self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
		self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_DIALOG)
		self.dialog.show_all()
		
class DialogRequest(Dialog): 
	''' Default request dialog '''  
	def __init__(self, text, title=None, longtext=False, returnfunc=None, 
				return_args=[], cancel_func=None, cancel_args=[]):
		''' init dialog
		
		text	-- The text that describes the error
		title	-- The window title
		longtext	-- boolean, if True, the text will be displayed in multi-line mode
		returnfunc	-- a method that will be called on gtk.RESPONSE_OK signal.
		If not set, the Dialog will do absolutely nothing!
		return_args	-- an optional sequence of agruments for the returnfunc
		'''		 
		Dialog.__init__(self, 
					text=text, 
					title=title, 
					longtext=longtext, 
					returnfunc=returnfunc, 
					return_args=return_args,
					cancel_func=cancel_func, 
					cancel_args=cancel_args)
		self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
		self.dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
		self.dialog.show_all()
		
class DialogRequestApply(Dialog):
	''' Default request dialog with apply button '''
	def __init__(self, text, title=None, longtext=False, returnfunc=None, return_args=[]):
		''' init dialog
		
		text	-- The text that describes the error
		title	-- The window title
		longtext	-- boolean, if True, the text will be displayed in multi-line mode
		returnfunc	-- a method that will be called on gtk.RESPONSE_OK or gtk.RESPONSE_APPLY signal.
		If not set, the Dialog will do absolutely nothing!
		return_args	-- an optional sequence of agruments for the returnfunc
		'''	   
		Dialog.__init__(self, text=text, title=title, longtext=longtext, returnfunc=returnfunc, return_args=return_args)
		self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
		self.dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		self.dialog.add_button(gtk.STOCK_APPLY, gtk.RESPONSE_APPLY)				 
		self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
		self.dialog.show_all()
		
class DialogRequestApplyClose(Dialog):
	def __init__(self, text, title=None, longtext=False, returnfunc=None, return_args=[]):
		''' init dialog
		
		text	-- The text that describes the error
		title	-- The window title
		longtext	-- boolean, if True, the text will be displayed in multi-line mode
		returnfunc	-- a method that will be called on gtk.RESPONSE_OK or gtk.RESPONSE_APPLY signal.
		If not set, the Dialog will do absolutely nothing!
		return_args	-- an optional sequence of agruments for the returnfunc
		'''		 
		Dialog.__init__(self, text=text, title=title, longtext=longtext, returnfunc=returnfunc, return_args=return_args)
		self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
		self.dialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
		self.dialog.add_button(gtk.STOCK_APPLY, gtk.RESPONSE_APPLY)		
		self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
		self.dialog.show_all()
		
class DialogGetValue(Dialog):
	''' A dialog to get a user input and return it to a given function. '''
	def __init__(self, text, returnfunc, title=None, initial_value=None):
		print 'DialogGetValue!'
		Dialog.__init__(self, text=text, title=title)
		self.returnfunc = returnfunc
		self.dialog.connect('response', self.on_dialog_universal_response)
		self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
		self.dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
		self.entryValue = gtk.Entry()
		if initial_value != None:
			self.entryValue.set_text(str(initial_value))
		self.dialog.vbox.pack_start(self.entryValue, expand=True, fill=True, padding=2)
		self.dialog.show_all()
		
	def on_dialog_universal_response(self, widget, response, *entry):	  
		if response == gtk.RESPONSE_APPLY or response == gtk.RESPONSE_OK:
			self.returnfunc(self.entryValue.get_text())
			time.sleep(0.1) 
			if response == gtk.RESPONSE_OK:
				widget.destroy ()
			return True		
		elif response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_CLOSE:
			widget.destroy ()
			return False			   
	   
	
	
	
	
	
	
	
	
	
	
	
	
	
