#!/usr/bin/env python
# encoding=utf-8

import os
import xml.etree.ElementTree as ET
import base64
import gobject
import string
import re
import urllib
import shutil
import tools

class Bookmarks:
	''' The main bookmarks tree '''
	def __init__(self, app, backup_warn_func=None):
		self._app = app
		self.root = ET.Element("Webmarks")
		self.tree = ET.ElementTree(self.root)
		if os.path.exists(self._app.Settings['webmarkfile']):
			try:
				self.tree = ET.parse(self._app.Settings['webmarkfile'])
				self.root = self.tree.getroot()
			except:
				backup = str(self._app.Settings['webmarkfile']) + '-bak'
				damaged = str(self._app.Settings['webmarkfile']) + '-damaged'
				shutil.copy(self._app.Settings['webmarkfile'], damaged)
				shutil.copy(backup, self._app.Settings['webmarkfile'])
				if os.path.exists(self._app.Settings['webmarkfile']):
					self.tree = ET.parse(backup)
					self.root = self.tree.getroot()
				#self.store()
				if backup_warn_func:
					backup_warn_func()
		
	def store(self):
		backup = str(self._app.Settings['webmarkfile']) + '-bak'
		shutil.copy(self._app.Settings['webmarkfile'], backup)
		self.tree.write(self._app.Settings['webmarkfile'], encoding="utf-8")

	def get_iterree(self):
		return self.iterparent(self.tree)
	
	def remove_tag_from_all(self, tag):
		for parent, child in self.get_iterree():
			#icon = None
			child.set('id', str(id)) 
			Bookmarkhelper(child).remove_tag(tag)
		self.store()
		
		
	def order_tree(self, treeElement):
		if treeElement.tag == 'webmark':
			return False
		container = treeElement.getchildren()
		webmarks = []
		dirs = []
		for elem in container:
			key = elem.get("name")
			if elem.tag == 'webmark':
				webmarks.append((key, elem))
			elif elem.tag == 'dir':
				dirs.append((key, elem))
		dirs.sort()
		webmarks.sort()
		data = dirs + webmarks
		container[:] = [item[-1] for item in data]		
		
	def fetch_all_favicons(self):
		''' DOESN'T WORK CORRECTLY YET, DON'T USE!!! '''
		if self.get_iterree() == None:
			return False
		fetchthreads = []
		for parent, child in self.get_iterree():
			if child != None:
				fetch_favicon = DelayedAction(self.__fetch_favicon_thread, child)
				fetch_favicon.start()
				fetchthreads.append(fetch_favicon)
		return
	
	def __fetch_favicon_thread(self, args):
		child = args[0]
		url = child.get('url')
		if url:
			favicon = self.fetch_favicon(url)
			if favicon != None:
				child.set('icon', favicon)
			else:
				child.set('icon', '')

	def fetch_favicon(self, url):
		pattern = re.compile('(http://|https://)([a-zA-Z0-9_\-\:\.\?\&\+]+)(\.{1}.*)(\/*.*)')
		match = pattern.search(url)
		domain = None
		iconData = None
		if match != None:
			domain = pattern.sub('\g<1>\g<2>\g<3>', match.group())
			domain += '/favicon.ico'
			filename, headers = urllib.urlretrieve(domain)
			try:
				filename, headers = urllib.urlretrieve(domain)
			except:
				return None
			f = open(filename, "r")
			iconBinaryData = f.read()
			if not '404' in iconBinaryData:
				HEADER = "data:image/bmp;base64,"
				iconData = HEADER + base64.encodestring(iconBinaryData)
			f.close()
		return iconData	
	
	def fetch_bookmark_title(self, url):
		opener = urllib.FancyURLopener({})
		try:
			f = opener.open(url)
		except:
			return str(url)
		site = f.read()
		pattern = re.compile('(\<title\>)(.+)(\</title\>)')
		match = pattern.search(site)
		pattern2 = re.compile('(\<meta name=\"title|TITLE|Title\" )(content=\")(.*)(\"{1})( *)(/*)(\>{1})')
		match2 = pattern2.search(site)
		titlestr = None
		if match != None:
			titlestr = pattern.sub('\g<2>', match.group())
		elif match2 != None:
			titlestr = pattern2.sub('\g<3>', match2.group())
		print 'titlestr: %s' % titlestr
		if titlestr:		
			html_entities = {'&nbsp;':' ','&mdash;':'--','&amp;':' '}
			for entity, replacement in html_entities.items():
				titlestr = titlestr.replace(entity, replacement)
		print 'titlestr2: %s' % titlestr
		return titlestr
	
	def fetch_bookmark_info(self, url):
		opener = urllib.FancyURLopener({})
		try:
			f = opener.open(url)
		except:
			return ''
		site = f.read()
		pattern = re.compile('(\<meta name=\"description|DESCRIPTION|Description\" )(content=\")(.*)("{1})( *)(/*)(\>{1})')
		match = pattern.search(site)
		titlestr = None
		if match != None:
			titlestr = pattern.sub('\g<3>', match.group())
			print 'titlestr: %s' % titlestr
			html_entities = {'&nbsp;':' ','&mdash;':'--','&amp;':' '}
			for entity, replacement in html_entities.items():
				titlestr = titlestr.replace(entity, replacement)
		return titlestr
		
	def iterparent(self, tree):
		root = tree.getroot()
		self._app.last_id = len(tree.getiterator())-1	   
		yield None, root
		for parent in tree.getiterator():
			for child in parent:
				yield parent, child
				
class Bookmarkhelper:
	def __init__(self, Element):
		''' Bookmark definition
		Element	-- A xml.etree.ElementTree.Element object. If given,
		the Bookmark will be initialized with it's values. Else the bookmark is
		considered as new.
		
		'''
		self.__Element = Element
		self.tags = []
		self.parent = None
		self.icon = None
	
	def add_tag(self, tag):
		''' Add the given String to the tags. '''
		tag = string.strip(tag)
		tags = self.__Element.get("tags")
		taglist = []
		if tags:
			taglist = string.split(self.__Element.get("tags"), ',')
		if tools.validate_tag(tag) != False:
			if tag not in taglist:
				taglist.append(tag)
			taglist.sort()
			self.__Element.set("tags", string.join(taglist, ','))
			
	def remove_tag(self, tag):
		''' Remove the given String to the tags. '''
		tag = string.strip(tag)
		try:
			tags = string.split(self.__Element.get("tags"), ',')
		except:
			return
		if tag in tags:
			tags.remove(tag)
			tags.sort()
			self.__Element.set("tags", string.join(tags, ','))
		return		   
	
	def get_id(self):
		return self.__Element.get("id") or ''
	
	def get_name(self):
		if self.__Element.get("name"):
			return self.__Element.get("name")
		return ''
	
	def get_name_markup(self):
		if self.__Element.get("name"):
			return gobject.markup_escape_text(self.__Element.get("name"))
		return ''		
	
	def get_url(self):
		if self.__Element.get("url"):
			return self.__Element.get("url")
		return ''
	
	def get_url_markup(self):
		if self.__Element.get("url"):
			return gobject.markup_escape_text(self.__Element.get("url"))
		return ''	
	
	def get_desc(self):
		if self.__Element.get("desc"):
			return self.__Element.get("desc")
		return ''
	
	def get_desc_markup(self):
		if self.__Element.get("desc"):
			return gobject.markup_escape_text(self.__Element.get("desc"))
		return ''	
	
	def get_tags(self):
		tags = self.__Element.get("tags")
		if tags:		
			return string.split(tags, ',') or []
	
	def get_tags_string(self):
		tags = self.__Element.get("tags")
		if tags:
			return tags.replace(',', ', ')
		return ''
	
	def get_tags_markup(self):
		return gobject.markup_escape_text(self.get_tags_string())
	
	def get_app(self):
		return self.__Element.get("app")  or ''
	
	def get_parent(self):
		return self.__Element.parent
	
	def get_binary_icon(self): 
		if self.__Element.get("icon"):
			HEADER = "data:image/bmp;base64,"
			iconData = self.__Element.get('icon')
			if iconData.startswith(HEADER):
				iconData = iconData[len(HEADER):]
				try:
					iconBinaryData = base64.decodestring(iconData)
					return iconBinaryData
				except:
					return None
		return None
	
	def get_overview(self):
		name = gobject.markup_escape_text(self.__Element.get("name"))
		markup = '<big><b>%s</b></big>\n\n' % name
		if self.__Element.get("desc") not in (None, ''):
			markup += '%s\n' % self.get_desc_markup()
		if self.__Element.get("url") not in (None, ''):
			markup += '<b>Url:</b> %s\n' % self.get_url_markup()
		if self.__Element.get("tags") not in (None, '', []):
			markup += '<b>Tags:</b>\n%s\n' % self.get_tags_markup()
		return markup or ''
	
	def get_overview_head(self):
		markup = ''
		if self.__Element.get("name"):
			markup = '<big><b>%s</b></big>' % self.get_name_markup()
		if self.__Element.get("url") not in (None, ''):
			markup += '\n<span>%s</span>' % self.get_url_markup()
		return markup
	
	def get_overview_body(self):
		markup = ''
		if self.__Element.get("desc") not in (None, ''):
			markup = '%s\n' % self.get_desc_markup()
		return markup
	
	def get_overview_footer(self):
		markup = '%s\n' % self.get_tags_markup()
		return markup
		
def validate_tag(tag):
	pattern = '([a-zA-Z0-9 _-]+)'
	match = re.match(pattern, tag)
	return bool(match)
		
			