#!/usr/bin/env python
# coding=utf-8

## A standard import plugin. 
## The file name for an import-plugin has always to begin with "import", folowed by any fancy name.
## Please have a look at the Plugins-list at createweb.de/en/webmarks to avoid name-collisions 
## with plugins that already exist!!! 
##
## There has always to be a main class called "Plugin" (case-sensitive).
## This class must include the method "get_tree" as described above.
#####
## If you write new Plugins, please leave all initial comments untouched and comment
## your own code as well, so that other developers may easily derive new plugins from it.
##Please register your plugin at createweb.de/en/webmarks, even
## If you'll publish it on your own site or elsewhere, to keep the naming of all 
## existing plugins consistent.
##
##    This plugin is free software; you can redistribute it and/or
##    modify it under the terms of the GNU Lesser General Public License
##    as published by the Free Software Foundation; either version 2
##    of the License, or (at your option) any later version.
##    
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##    
##    You should have received a copy of the GNU Lesser General Public License
##    along with this program; if not, write to the Free Software
##    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import base64
import re
import os
import xml.etree.ElementTree as ET
from BeautifulSoup import BeautifulSoup
import gettext
import webmarx

## get the gettext translation from webmarx.
## NEVER REMOVE THIS, ELSE TRANSLATIONS WILL BE BROKEN!
_ = webmarx._

class Plugin:
	''' An import plugin for firefox-bookmarks.
	
	'''
	def __init__(self):
		''' Initialise the plugin.
		
		bookmarks_file	Must be an absolute path to the file to import.

		'''
		self.name = _('Firefox 2-3 import')
		self.description = _('Imports all bookmarks from original firefox bookmarks.html file.')
		self.enabled = True
		self.haveconfig = False

		self.root = ET.Element("Webmarks")
		self.etree = ET.ElementTree(self.root)
		self.root.parent = None		

	def get_tree(self, bookmarks_file):
		infile = os.path.abspath(bookmarks_file)
		fd = file(infile)
		page = BeautifulSoup(fd, selfClosingTags=['dd'])   
		self.iterchildren(page, self.root)
		return self.etree
	
	def get_name(self):
		return self.name
	
	def get_description(self):
		return self.description

	def iterchildren(self, soup, etree_element):
		for dl_tag in soup.findAll("dl", recursive=False):
			for dt_tag in dl_tag.findAll("dt", recursive=False):
				if dt_tag.h3:
					et_child = ET.Element("dir")
					et_child.set('name', dt_tag.h3.string)				
					self.iterchildren(dt_tag, et_child)
					etree_element.append(et_child)
					#ET.dump(et_child)
				elif dt_tag.a:
					et_child = ET.Element("webmark")
					if dt_tag.a["href"] == None:
						pass
					else:
						name = dt_tag.a.string
						if name == None:
							name = dt_tag.a["href"]
						et_child.set('name', name)
						et_child.set('url', dt_tag.a["href"])
						try:
							et_child.set('icon', dt_tag.a["icon"])
						except:
							pass
						etree_element.append(et_child)
		return
	
	def open_config(self):
		''' This method may be used to open a configuration-dialog for your plugin. 
		If a configuration is needed for your plugin, you have to write your own
		gtk-gui that will be opened by this method. You also have to set up your 
		own configuration-mechanism. There is no standardized way for this.
		If you want to use a configuration, set the parameter haveconfig in __init__
		to True, to show the config-button in the settings-dialog.
		
		'''
		return None	





	