#!/usr/bin/env python
# coding=utf-8

## A standard launcher plugin. 
## The file name for a launcher-plugin has always to begin with "launch", folowed by any fancy name.
## Please have a look at the Plugins-list at createweb.de/en/webmarks to avoid name-collisions 
## with plugins that already exist!!! 
##
## There has always to be a main class called "Plugin" (case-sensitive).
## This class must include the methods "launch" and "get_name" as described above.
## The Launch command must be run as thread, or independent process,
## otherwise the entire program will freeze until the launched application is closed!
######
## If you write new Plugins, please leave all initial comments untouched and comment
## your own code as well, so that other developers may easily derive new plugins from it.
##Please register your plugin at createweb.de/en/webmarks, even
## If you'll publish it on your own site or elsewhere, to keep the naming of all 
## existing plugins consistent.
##
##	This plugin is free software; you can redistribute it and/or
##	modify it under the terms of the GNU Lesser General Public License
##	as published by the Free Software Foundation; either version 2
##	of the License, or (at your option) any later version.
##	
##	This program is distributed in the hope that it will be useful,
##	but WITHOUT ANY WARRANTY; without even the implied warranty of
##	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##	GNU General Public License for more details.
##	
##	You should have received a copy of the GNU Lesser General Public License
##	along with this program; if not, write to the Free Software
##	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import threading
import webbrowser
import gettext
import webmarx

## get the gettext translation from webmarx.
## NEVER REMOVE THIS, ELSE TRANSLATIONS WILL BE BROKEN!
_ = webmarx._

class Plugin:
	''' A standard launcher plugin.
	
	'''
	def __init__(self):
		''' Initialise the plugin.
		
		name is the string that will be displayed on the launch-button.
		it should always be in brackets with a leading underline to be
		translatable. You may call this attribute other than name 
		or even generate it from a function, as long as
		you provide the proper string through the "get_name" method.
		
		'''
		#self.__id = _('Default browserLauncher1')
		self.name = _('Default browser')
		self.description = _('Open the bookmark with your systems default web browser.\n\n \
		Only works with protocols that are supported by your browser like http or ftp.')
		self.enabled = True
		self.haveconfig = False
				
		self.show_button = True # Show the launch button for this plugin by default
		self.show_edit_combo = True # Show the entry in the editors combobox-default-app for this plugin by default		
		
	def launch(self, url):
		''' This method is mandatory for all launcher plugins. 
		it has always to receive the url as 2. argument.
		
		Return	boolean True | False (mandatory!)
		
		'''
		if url == '':
			return False
		browser = DefaultBrowser(url)
		browser.start()
		return True
	
	def get_name(self):
		''' This method is mandatory for all launcher plugins. 
		it has always to return the Name to be shown on the button as 
		translatable string (see above in __init__). The name has also to be unique 
		within to all installed plugins!!!
		'''
		return self.name
	
	def get_tooltip(self):
		''' This method is mandatory for all launcher plugins. 
		it has always to return a tooltip text for the button as 
		translatable string (see above in __init__), or None
		'''
		return self.description
	
	def open_config(self):
		''' NOT IMPLEMENTED YET FOR LAUNCHER PLUGINS!!!
		This method may be used to open a configuration-dialog for your plugin. 
		If a configuration is needed for your plugin, you have to write your own
		gtk-gui that will be opened by this method. You also have to set up your 
		own configuration-mechanism. There is no standardized way for this.
		If you want to use a configuration, set the parameter haveconfig in __init__
		to True, to show the config-button in the settings-dialog.
		
		'''
		return None	
	
class DefaultBrowser (threading.Thread):
	''' This is a fancy function to launch the default browser.
	It does not belong to the general Plugin structure.
	'''
	def __init__ (self, url):
		threading.Thread.__init__(self)
		self.url = url
		self.target = "default"
	
	def run(self):
		self.open_website(self.url, self.target)  
	
	def open_website(self, url, target):
		if target == "new window":
			newin = 1
		else:
			newin = 0
		#runweb = 
		webbrowser.open(self.url, new=newin)
		#print 'runweb: %s' % runweb
		return
