#!/usr/bin/env python
# encoding=utf-8

__author__    = "Rainer Haage <webmarx@createweb.de>"
__version__   = "1.0"
__date__      = "2009-01-10"
__copyright__ = "Copyright (c) 2009-2012 %s. All rights reserved." % __author__
__licence__   = "GPL"
__url__   = "webmarx.createweb.de"

import os
import sys
import pickle
import shutil
import gettext
import locale

mswindows = (sys.platform[:3] == 'win')

basedir = '.'
try:
	basedir = os.path.dirname(os.path.realpath(__file__))
except:
	if hasattr(sys, 'frozen'):
		basedir = os.path.split(sys.executable)[0]
	else:
		basedir = os.path.split(sys.argv[0])[0] 
if os.path.exists(basedir) and basedir not in sys.path:
	sys.path.insert(0, basedir)

class Settings:
	#global prefix
	''' The main bookmarks tree '''
	def __init__(self, app):
		self._app = app   
		plugindir = os.path.join( basedir, 'plugins')
		if hasattr(sys, 'frozen'):
			plugindir = os.path.join( self._app.prefix, 'plugins')
			
		self.__settings = {'usetray' : True
						   ,'close_to_tray' : True
						   ,'close_on_launch' : True
						   ,'autofetch' : True
						   ,'default_launcher' : _('Default application')
						   #,'iconsize' : 16
						   ,'iconsize_tree' : 16
						   ,'iconsize_menu' : 16
						   ,'iconsize_overview' : 32
						   ,'launchers' : []
						   }
		self.__system_settings = {'basedir': basedir
						   ,'pixmapdir' : os.path.join( self._app.prefix, 'share', 'webmarx')
						   ,'plugindirs' : [plugindir,]
						   ,'libdir' : os.path.join( self._app.prefix, 'share', 'webmarx')
						   ,'docdir' : os.path.join( self._app.prefix, 'doc', 'webmarx')
						   ,'compactsize' : 360
						   }
				
		if sys.platform[:3] == 'win':
			self.init_win()
		elif sys.platform[:3] == 'lin':  
			self.init_lin()
		else:
			try:		
				self.init_lin()
			except:
				print "Your os: '%s' is not supported yet. If you are interested in\
		getting a version for your os, please contact us: http://www.createweb.de" % str(sys.platform)
				sys.exit(1)   
		self.__system_settings['webmarkfile'] = os.path.join(self.__system_settings['configdir'], 'webmarks.xml')
		self.__system_settings['tagfile'] = os.path.join(self.__system_settings['configdir'], 'tags.pickle')
		
		if not os.path.exists(self.__system_settings['configdir']):
			os.mkdir(self.__system_settings['configdir'])
			example_file = os.path.join(self.__system_settings['libdir'], 'webmarks.xml')
			example_tags = os.path.join(self.__system_settings['libdir'], 'tags.pickle')
			shutil.copy(example_file, self.__system_settings['configdir'])
			shutil.copy(example_tags, self.__system_settings['tagfile'])
		self.__settings.update(self.__system_settings)	   
		self.configfile = os.path.join(self.__system_settings['configdir'], 'settings.pickle')
		for path in self.__system_settings['plugindirs']:
			if not os.path.exists(path):
				try:
					os.mkdir(path)
				except:
					pass			
		
		
	def init_win(self):
		''' This has to be called if the program runs on windows systems. 

		'''
		class NullDevice:
			def write(self, s):
				pass
		if hasattr(sys, 'frozen'):
			sys.stderr = NullDevice()
		import pywintypes
		import pythoncom
		import win32api		
		import _winreg
		appdir = os.environ['APPDATA']
		self.__system_settings['configdir'] = os.path.join(appdir, 'webmarx')
		#self.__system_settings['libdir'] = os.path.join(appdir, 'webmarx')
		self.__system_settings['plugindirs'].append(os.path.join(appdir, 'webmarx', 'plugins'))
		
	def init_lin(self):
		''' This has to be called if the program runs on linux or other *nix systems. 
		
		Allways call this method BEFORE!!! loading the settings from database
		in order to avoid post-processing of loaded variables through this method!
		'''
		homedir = os.path.expanduser('~')
		self.__system_settings['configdir'] = os.path.join(homedir, '.webmarx')
		self.__system_settings['plugindirs'].append(os.path.join(homedir, '.webmarx', 'plugins'))
		
	def store(self):
		dumpf = open(self.configfile , "w")		
		pickle.dump(self.__settings, dumpf)
		dumpf.close()
	   
	def load(self):
		try:
			dumpf = open(self.configfile , "r")	  
			__settings = pickle.load(dumpf)
			dumpf.close()
			self.__settings.update(__settings)
			self.__settings.update(self.__system_settings)
			return self.__settings
		except:
			return None
		return None
	
	def __getattr__(self, attr):
		try:
			return self.__settings[str(attr)]
		except:
			return
		
	def __getitem__(self, item):
			return self.__settings[str(item)]
		
	def __setitem__(self, item, value):
		self.__settings[str(item)] = value