#!/usr/bin/env python
# encoding=utf-8

##	Qemulator - a qemu gui written in python and GTK/Glade.
##	Copyright (C) 2006  rainer haage
##
##	This program is free software; you can redistribute it and/or
##	modify it under the terms of the GNU General Public License
##	as published by the Free Software Foundation; either version 2
##	of the License, or (at your option) any later version.
##	
##	This program is distributed in the hope that it will be useful,
##	but WITHOUT ANY WARRANTY; without even the implied warranty of
##	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##	GNU General Public License for more details.
##	
##	You should have received a copy of the GNU General Public License
##	along with this program; if not, write to the Free Software
##	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
APP = 'Webmarks'

try:
	import pygtk
	pygtk.require("2.0")
	
except:
	  print "pygtk 2.0 is not installed"
try:
	import gtk
	import gtk.glade
	
except:
	print "gtk and/or glade is not installed - exiting..."
	sys.exit(1)
	
import sys
import os
from os import path
import gobject
import string
import time
import threading
from threading import Thread
	
try:
	import gettext
	import locale
except:
	print "locale and/or gettext for language support not installed"
	sys.exit(1)
	
from StaticPlugins import PluginLaunchbutton
from bookmarks import Bookmarkhelper

#class __Tray ( Thread ):
class __Tray ( gobject.GObject ):	
	stopthread = threading.Event()
				
	def __init__ (self, app):

		gobject.GObject.__init__(self)
		self._app = app
		self.traymenu_item = {}
		self.gladefile = self._app.Settings.libdir + "/Webmarks.glade"
		self.appendedParents = []
		self.default_icon = gtk.gdk.pixbuf_new_from_file(os.path.join(self._app.Settings.pixmapdir, 'www.png'))
		self.dir_icon = gtk.gdk.pixbuf_new_from_file(os.path.join(self._app.Settings.pixmapdir, 'dir-closed.png'))		
		
	def run(self):
		#self.stopthread.clear()
		self.init_tray()

	def init_tray(self):
		#gtk.gdk.threads_enter()
		self.wTree_tray_menu = gtk.glade.XML(self.gladefile,"tray_menu", APP)
		self.tray_menu = self.wTree_tray_menu.get_widget("tray_menu")
		self.traymen_item_exit = self.wTree_tray_menu.get_widget("item_exit")
		self.item_add_bookmark = self.wTree_tray_menu.get_widget("item_add_bookmark")
		
		self.dic_menu_tray = {"on_item_exit_activate" : self._app.exit,
							"on_item_add_bookmark_activate" : self.on_item_add_bookmark_activate
		}
		self.wTree_tray_menu.signal_autoconnect(self.dic_menu_tray) 
		self.tray_icon = gtk.StatusIcon()
		icon_file = self._app.Settings.pixmapdir + "/trayicon.png"
		self.tray_icon.set_from_file(icon_file)
		self.tray_icon.set_tooltip('WebMarX')
		
		self.tray_menu.set_title("WebMarX")
		self.tray_menu.append(gtk.SeparatorMenuItem()) 
		
		self.traymen_item_bookmarks = gtk.MenuItem(label=_('Webmarks'), use_underline=True)
		self.tray_menu.append(self.traymen_item_bookmarks)
		self.reset_menu()
		self.tray_menu.show_all()	  
		self.tray_icon.connect('popup-menu', self.on_popup_menu)
		self.tray_icon.connect("activate", self.on_tray_button_press_event)
		#gtk.gdk.threads_leave()
		
	def on_item_add_bookmark_activate(self, widget):
		#if not self._app.Window.mainwindow.get_property('visible'):
		self._app.Window.mainwindow.present()		
		self._app.Window.add_bookmark()
		
	def reset_menu(self):
		root_menu = gtk.Menu()
		self.traymen_item_bookmarks.set_submenu(root_menu)
		self.fill_root_menu(root_menu)	

		
	def fill_root_menu(self, menu):
		tooltips = gtk.Tooltips()
		tooltips.enable()
		self.fill_menu(menu, tooltips, self._app.Bookmarks.root)

	def fill_menu(self, menu, tooltips, contents):
		if len(list(contents)) == 0:
			menu_item = gtk.MenuItem("No Bookmarks")
			menu_item.set_sensitive(False)
			menu.append(menu_item)
		else:
			for child in list(contents):
				name = child.get('name')
				if name == None:
					menu_item = gtk.SeparatorMenuItem()
				else:
					
					menu_item = gtk.MenuItem()
					if child.tag == 'dir':
						icon = self.dir_icon
						folder = child
						sub_menu = gtk.Menu()
						menu_item.set_submenu(sub_menu)
						self.fill_menu(sub_menu, tooltips, folder)
					else:
						href = child.get('url')
						tooltips.set_tip(menu_item, href)
						menu_item.connect("activate", self.launch_item, child)
						iconBinaryData = None
						try:
							iconBinaryData = Bookmarkhelper(child).get_binary_icon()
						except:
							iconBinaryData = None
						if iconBinaryData != None:
							try:								
								loader = gtk.gdk.PixbufLoader()								
								loader.write(iconBinaryData)
								loader.close()
								icon = loader.get_pixbuf()
							except:
								try:
									iconFile, ipath = mkstemp(suffix='.bmp')
									os.write(iconFile, iconBinaryData)
									os.close(iconFile)
									icon = gtk.gdk.pixbuf_new_from_file(ipath)
								except:								
									icon = self.default_icon
							if icon == None:
								icon = self.default_icon
						else:
							icon = self.default_icon
												
					pwidth = icon.get_width()
					pheight = icon.get_height()
					pfact = float(pwidth) / float(pheight)
					oheight = float(self._app.Settings.iconsize_menu) / pfact
					icon = icon.scale_simple(int(self._app.Settings.iconsize_menu), int(oheight), 3)
					image = gtk.Image()
					image.set_from_pixbuf(icon)
					label = gtk.Label(name)
					#label.set_justify(gtk.JUSTIFY_LEFT)
					label.set_alignment(0.0, 0.5)
					hbox = gtk.HBox()
					hbox.pack_start(image, expand=False, fill=True, padding=2)
					hbox.pack_start(label, expand=True, fill=True, padding=2)
					menu_item.add(hbox)
				menu.append(menu_item)
			tearoffitem = gtk.TearoffMenuItem()
			menu.prepend(tearoffitem)				
				
	def launch_item(self, widget, treeElement):
		Plugins = self._app.launcherPlugins
		plugin = None
		default_launcher = None
		for Plugin in Plugins:
			if treeElement.get('app') == Plugin.get_name():
				plugin = Plugin	
				break
			if self._app.Settings.default_launcher == Plugin.get_name():
				default_launcher = Plugin			
		if plugin != None:
			plugin.launch(treeElement.get('url'))
			return True

		launchers = self._app.Settings.launchers
		for name, command in launchers:
			Plugin = PluginLaunchbutton(name, command)
			if treeElement.get('app') == Plugin.get_name():
				plugin = Plugin
				break
			if self._app.Settings.default_launcher == Plugin.get_name():
				default_launcher = Plugin
		if plugin != None:
			plugin.launch(treeElement.get('url'))
			return True
		else:
			default_launcher.launch(treeElement.get('url'))
			return True			
		
	def on_popup_menu(self, *args):
		self.tray_menu.show_all()
		self.tray_menu.popup(None, None, None, 0, 0)
		
		return 1
	
	def resize_popup(self, widget=None, *event):
		#print "resizing popup"
		pass

	def on_tray_button_press_event(self, widget):
		if self._app.Window.mainwindow.get_property('visible'):
			self._app.Window.mainwindow.hide()
		else:
			#if event.type == gtk.gdk._2BUTTON_PRESS:
			self._app.Window.mainwindow.present()
		
__tray = None

def tray(app):
	global __tray   
	if __tray == None:
		settings = app.Settings
		if settings.usetray == True:		 
			__tray = __Tray(app)
			#__tray.start()
			__tray.run()
		else:
			return None   
	return __tray
					